# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/1
import os
import json
import inspect
from yqn_project_cli.rpc.eureka import eureka_client
from yqn_project_cli.rpc.apollo.apollo_client import ApolloClient
from yqn_project_cli.rpc import RPCConfigBase
from yqn_project_cli.utils import host_ip


class ApolloConfig(RPCConfigBase):

    def __init__(self, **kwargs):
        super(ApolloConfig, self).__init__(**kwargs)

        self.apollo_app_id = self.client_config.apollo_app_id
        self.apollo_app_name = self.client_config.apollo_app_name
        self.apollo_cluster = os.environ.get('YQN_APOLLO_CLUSTER', self.client_config.apollo_cluster)
        self.apollo_meta_server_url = os.environ.get('YQN_APOLLO_METADATA', self.client_config.apollo_meta_server_url)
        self.apollo_env = os.environ.get('YQN_APOLLO_ENV', self.client_config.apollo_env)
        self.apollo_namespace = getattr(self.client_config, 'apollo_namespace', 'application')

        self.apollo_eureka_server_key = getattr(self.client_config, 'apollo_eureka_server_key',
                                                'eureka.client.serviceUrl.defaultZone')

        self.apollo_eureka_status_page_url = getattr(self.client_config, 'apollo_eureka_status_page_url',
                                                     self.project_config.heartbeat_url)

        self.apollo_eureka_health_check_url = getattr(self.client_config, 'apollo_eureka_health_check_url',
                                                      self.project_config.heartbeat_url)

        self.apollo_eureka_instance_ip = getattr(self.client_config, 'apollo_eureka_instance_ip',
                                                 os.environ.get('YQN_POD_IP', host_ip()))

        self.apollo_eureka_instance_port = getattr(self.client_config, 'apollo_eureka_instance_port',
                                                   int(self.apollo_app_id))

        self.apollo_cache_file_path = getattr(self.client_config, 'apollo_cache_file_path',
                                              os.path.join(self.project_config.project_base_dir, 'config'))

        self.conf_management = {}
        self.services = {}

    def init_apollo(self):
        apollo_client = ApolloClient(app_id=self.apollo_app_id,
                                     cluster=self.apollo_cluster,
                                     config_server_url=self.apollo_meta_server_url,
                                     env=self.apollo_env,
                                     cache_file_path=self.apollo_cache_file_path)

        if self.project_config is not None and getattr(self.project_config, 'hot_update', False):
            apollo_client.start(lambda: self.manage_servers(apollo_client))  # hot update

        else:
            apollo_client.start(lambda: self.reload(apollo_client))

        self.connector = apollo_client

    def init_eureka(self):
        eureka_server = self.connector.get_value(self.apollo_eureka_server_key, self.connector.get_value('eureka_url'))
        eureka_client.init(eureka_server=eureka_server,
                           app_name=self.apollo_app_name,
                           instance_ip=self.apollo_eureka_instance_ip,
                           instance_port=self.apollo_eureka_instance_port,
                           status_page_url=self.apollo_eureka_status_page_url,
                           health_check_url=self.apollo_eureka_health_check_url)

    def manage_servers(self, apollo_client):
        updated = self.reload(apollo_client)
        if updated:
            for name, server in self.services.items():
                if server is self or not callable(getattr(server, 'reconnect', False)):
                    continue

                # server reconnect depend on certain conf_management attrs
                server.reconnect(**self.conf_management)

    def reload(self, apollo_client):

        # va
        # if 'certain_self_settings_namespace' in apollo_client._cache:
        #     self.conf_management = apollo_client._cache['certain_self_settings_namespace']
        #
        # else:
        #     self.conf_management = apollo_client._cache['application']

        # vb
        updated = False
        if self.conf_management != apollo_client._cache[self.apollo_namespace]:
            self.conf_management = apollo_client._cache[self.apollo_namespace]
            updated = True

        return updated

        # vc
        # for key, value in inspect.getmembers(ApolloConfigMapping):
        #     if not key.startswith('__') and not callable(value):
        #         self.conf_management[key] = apollo_client.get_value(key)

        # vd
        # self.conf_management = dict(
        #
        #     eureka_url=apollo_client.get_value('eureka_url'),
        #
        #     es_host=apollo_client.get_value('es_host'),
        #     es_port=apollo_client.get_value('es_port'),
        #     es_username=apollo_client.get_value('es_username'),
        #     es_password=apollo_client.get_value('es_password'),
        #
        #     mysql_engine=apollo_client.get_value('mysql_engine'),
        #     mysql_username=apollo_client.get_value('mysql_username'),
        #     mysql_password=apollo_client.get_value('mysql_password'),
        #     mysql_host=apollo_client.get_value('mysql_host'),
        #     mysql_port=apollo_client.get_value('mysql_port'),
        #     mysql_charset=apollo_client.get_value('mysql_charset'),
        #     mysql_database=apollo_client.get_value('mysql_database'),
        #     mysql_url=apollo_client.get_value('mysql_url'),
        #
        #     odps_access_id=apollo_client.get_value('odps_access_id'),
        #     odps_secret_access_key=apollo_client.get_value('odps_secret_access_key'),
        #     odps_project=apollo_client.get_value('odps_project'),
        #     odps_endpoint=apollo_client.get_value('odps_endpoint'),
        #
        # )

    def connect(self, **kwargs):
        self.init_apollo()
        self.init_eureka()
        return self.conf_management

    def close(self, **kwargs):
        pass

    def reconnect(self, **kwargs):
        pass



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/24
import os
import sys
import json
import os.path as op
import datetime
import inspect
import platform
import importlib

methods = ('get', 'post', 'put', 'patch', 'delete', 'head', 'options', 'trace')

view_cls_template = '''

class {view_cls}:
    """{view_cls}'s doc"""
'''

view_mth_template = '''
    def {view_mth}(self, *args, **kwargs):
        """{doc}"""
        return JSONResponse()
'''

empty_init = """
# -*- coding: utf-8 -*-
# Author: {author}
# Date：{_date}
""".format(author=(platform.node() or 'SOMEBODY').upper(), _date=datetime.date.today().strftime('%Y/%m/%d')).lstrip()

empty_handler = empty_init

empty_model = empty_init.rstrip() + """
from yqn_project_cli.utils.core.request import RequestSchemaBase
from yqn_project_cli.utils.core.response import ResponseSchemaBase
"""
empty_parser = empty_init.rstrip() + """
from yqn_project_cli.utils.core.parser import (
    DAArgument,
    DARequestParser
)
'''

class DemoParser(DARequestParser):
    arg1 = DAArgument('arg1', required=True, type=int, location='args')
    arg2 = DAArgument('arg2', required=True, type=str, location='args')
'''

"""
empty_views = empty_init.rstrip() + """
from yqn_project_cli.utils.core.as_flask import JSONResponse
from yqn_project_cli.utils.restx import DANamespace

# one module, one namespace, more will be ignored
ns = DANamespace('{module}', description='', path='/')
"""

empties = {
    '__init__.py': empty_init,
    'handler.py': empty_handler,
    'model.py': empty_model,
    'parser.py': empty_parser,
    'views.py': empty_views,
}


def create_module(path, module_name):
    os.makedirs(op.join(path, module_name), exist_ok=False)

    for file, content in empties.items():
        if file == 'views.py':
            content = content.format(module=module_name)

        with open(op.join(path, module_name, file), 'w') as writer:
            writer.write(content)


def add_2_file(file_path, route):
    dst = view_cls_template.format(view_cls=route['view_cls'])

    if route.get('view_mth', False):
        dst += view_mth_template.format(view_mth=route['view_mth'], doc=route['doc'])

    else:
        raise ValueError('%s.%s must defined' % (route['view_cls'], route['view_mth']))

    with open(file_path, 'a') as writer:
        writer.write(dst)


def update_2_file(file_path, views, route):
    src = dst = inspect.getsource(getattr(views, route['view_cls']))

    if route.get('view_mth', False):
        if not getattr(getattr(views, route['view_cls']), route['view_mth'], False):
            dst += view_mth_template.format(view_mth=route['view_mth'], doc=route['doc'])
    else:
        raise ValueError('%s.%s must defined' % (route['view_cls'], route['view_mth']))

    if dst != src:
        with open(file_path, 'r') as reader:
            read_str = reader.read()

        read_str = read_str.replace(src, dst)

        with open(file_path, 'w') as writer:
            writer.write(read_str)


def auto_view(config, init=False):
    if op.join(config['app_path'], config['app_name']) not in sys.path:
        sys.path.append(op.join(config['app_path'], config['app_name']))

    routes = json.load(open(op.join(config['app_path'], config['app_name'],
                                    'config/{}.json'.format(config['app_name'])), 'rb'))

    for route in routes['path_list']:
        # not existed view_cls
        assert isinstance(route, dict), "path_list中的值需为对象"
        assert all([route["module"], route["view_cls"], route["http_methods"]]), \
            "module、view_cls、http_methods需为bool转换后为真数据"

        # create new module
        if not op.exists(op.join(config['app_path'], config['app_name'], 'api', route['module'])):
            create_module(op.join(config['app_path'], config['app_name'], 'api'), route['module'])

        file_path = op.join(config['app_path'], config['app_name'], 'api', route['module'], 'views.py')

        # init project by filling in views.py based on json-config
        if init:
            add_2_file(file_path, route)

        else:

            api_module = importlib.import_module('api')
            importlib.reload(api_module)

            views = importlib.import_module('api.{}.views'.format(route['module']))
            importlib.reload(views)

            if not getattr(views, route['view_cls'], False):
                add_2_file(file_path, route)

            else:
                update_2_file(file_path, views, route)

    if op.join(config['app_path'], config['app_name']) in sys.path:
        sys.path.pop(sys.path.index(op.join(config['app_path'], config['app_name'])))



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/24
import os
from yqn_project_cli.utils import (
    conf_loader,
    host_ip,
    host_name,
)


class ProjectConfig:
    id = "${app_id}"
    name = "${project_name}"

    version = '1.0.0'
    version_int = 1 * 10000 + 0 * 100 + 0 * 1

    heartbeat_url = '/actuator/info/'
    heartbeat_urls = [heartbeat_url, '/actuator/prometheus/', '/info/']

    project_base_dir = os.path.dirname(os.path.dirname(__file__))
    project_father_dir = os.path.dirname(project_base_dir)

    resources = os.path.join(project_base_dir, 'resources')  # resource_path
    outputs = os.path.join(project_base_dir, 'outputs')  # output_path

    hot_update = False  # whether service hot-update or not

    timer_logger_over_secs = 0  # disabled if False; must logger if None or 0; logger when greater than positive nums
    timer_logger_skip_endpoints = {'doc', 'specs', 'restx_doc.static', 'static'}  # set-typed
    timer_logger_type = 'timer'


class LoggerConfig:
    app_id = ProjectConfig.id
    app_name = ProjectConfig.name
    log_path = '/app/log/'  # log_path = '/app/' + app_name + '/log/'

    os.makedirs(log_path, exist_ok=True)


local_conf_loader = conf_loader(
    os.path.join(ProjectConfig.project_father_dir, '.{}_config.py'.format(ProjectConfig.name)))


class ApolloClientConfig:
    apollo_cache_file_path = os.path.join(ProjectConfig.project_base_dir, 'config')  # apollo cached config-file path
    apollo_cluster = local_conf_loader('YQN_APOLLO_CLUSTER', "qa")
    apollo_meta_server_url = local_conf_loader('YQN_APOLLO_METADATA', 'http://192.168.10.227:8080')
    apollo_env = local_conf_loader('YQN_APOLLO_ENV', "FAT")
    apollo_namespace = local_conf_loader('APOLLO_NAMESPACE', "application")

    apollo_app_name = local_conf_loader('APP_NAME', ProjectConfig.name)
    apollo_app_id = local_conf_loader('APP_ID', ProjectConfig.id)

    os_node_ip = local_conf_loader('YQN_NODE_IP', host_ip())
    os_pod_ip = local_conf_loader('YQN_POD_IP', host_ip())
    os_pod_name = local_conf_loader('YQN_POD_NAME', host_name())

    # !important set in apollo platform, like key 'eureka.client.serviceUrl.defaultZone'
    apollo_eureka_server_key = 'eureka_url'
    apollo_eureka_status_page_url = ProjectConfig.heartbeat_url
    apollo_eureka_health_check_url = ProjectConfig.heartbeat_url
    apollo_eureka_instance_ip = os_pod_ip
    apollo_eureka_instance_port = int(apollo_app_id)


class ODPSConfig:
    odps_access_id = local_conf_loader('ODPS_ACCESS_ID', '')
    odps_secret_access_key = local_conf_loader('ODPS_SECRET_ACCESS_KEY', '')
    odps_project = local_conf_loader('ODPS_PROJECT', '')
    odps_endpoint = local_conf_loader('ODPS_ENDPOINT', '')


class MySQLConfig:
    mysql_engine = local_conf_loader('MYSQL_ENGINE', '')
    mysql_username = local_conf_loader('MYSQL_USERNAME', '')
    mysql_password = local_conf_loader('MYSQL_PASSWORD', '')
    mysql_host = local_conf_loader('MYSQL_HOST', '')
    mysql_port = local_conf_loader('MYSQL_PORT', '')
    mysql_charset = local_conf_loader('MYSQL_CHARSET', '')
    mysql_database = local_conf_loader('MYSQL_DATABASE', '')


class ESConfig:
    es_host = local_conf_loader('ES_HOST', '')
    es_port = local_conf_loader('ES_PORT', '')
    es_username = local_conf_loader('ES_USERNAME', '')
    es_password = local_conf_loader('ES_PASSWORD', '')


class RedisConfig:
    redis_host = local_conf_loader('REDIS_HOST', '')
    redis_port = local_conf_loader('REDIS_PORT', '')
    redis_db = local_conf_loader('REDIS_DB', '')
    redis_password = local_conf_loader('REDIS_PASSWORD', '')



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/24
import json
from werkzeug.exceptions import HTTPException
from flask import request


class BasicException(BaseException):
    """for apollo exceptions"""

    def __init__(self, msg: str):
        self._msg = msg
        print(msg)

    def __str__(self):
        return "%s: %s" % (self.__name__, self._msg)


class NameSpaceNotFoundException(BasicException):
    """for apollo exceptions"""


class ServerNotResponseException(BasicException):
    """for apollo exceptions"""


# class _Exception(HTTPException):
#     errmsg = None
#     errcode = None
#     data = None
#     code = None
#
#     def __init__(self, errmsg=None, errcode=None, data=None, code=None):
#         super(_Exception, self).__init__(errmsg, None)
#
#         if errmsg is not None:
#             self.errmsg = errmsg
#
#         if errcode is not None:
#             self.errcode = errcode
#
#         if data is not None:
#             self.data = data
#
#         if code is not None:
#             self.code = code
#
#     def get_body(self, environ=None):
#         body = dict(
#             errcode=self.errcode,
#             errmsg=self.errmsg,
#             data=self.data
#         )
#         text = json.dumps(body, sort_keys=False, ensure_ascii=False)
#
#         return text
#
#     def get_headers(self, environ=None):
#         return [('Content-Type', 'application/json')]
#
#
# class APIException(_Exception):
#     code = 400
#     errmsg = 'sorry, request error'
#     errcode = "-1"
#
#
# class PortalException(_Exception):
#     code = 500
#     errmsg = 'sorry, internal error'
#     errcode = "-1"

class _Exception(HTTPException):
    data = None
    bizExtMap = None
    code = None
    header = {}
    headers = {}
    msg = ""
    msgCode = ""
    msgDetail = ""
    success = False

    def __init__(self, msgDetail="", data=None, bizExtMap=None, code=None, header=None,
                 headers=None, msg="", msgCode="", success=False, **kwargs):

        super(_Exception, self).__init__(msgDetail, None)

        self.data = data
        self.bizExtMap = bizExtMap

        if code is not None:
            self.code = code

        if header is None:
            req_data = request.json

            if isinstance(req_data, dict) and 'header' in req_data and isinstance(req_data['header'], dict):
                self.header = {
                    "xAppId": req_data['header'].get("xAppId", ""),
                    "xTraceId": req_data['header'].get("xTraceId", "")
                }
        else:
            self.header = header

        self.headers = headers
        self.msg = msg
        self.msgCode = msgCode
        self.msgDetail = msgDetail
        self.success = success
        self.kwargs = kwargs

    def get_body(self, environ=None):
        body = {
            "data": self.data,
            "bizExtMap": self.bizExtMap,
            "code": self.code,
            "header": self.header,
            "headers": self.headers,
            "msg": self.msg,
            "msgDetail": self.msgDetail,
            "msgCode": self.msgCode,
            "success": self.success,
            **self.kwargs,
        }

        text = json.dumps(body, sort_keys=False, ensure_ascii=False)

        return text

    def get_headers(self, environ=None):
        return [('Content-Type', 'application/json')]


class APIException(_Exception):
    code = 400
    msgDetail = 'sorry, request error'
    msgCode = "-1"


class PortalException(_Exception):
    code = 500
    msgDetail = 'sorry, internal error'
    msgCode = "-1"



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/13
from yqn_project_cli.utils.core.request import RequestSchemaBase
from yqn_project_cli.utils.core.response import ResponseSchemaBase


class RequestSchema(RequestSchemaBase):

    def get_model_schema(self):
        model = {'type': 'object', 'properties': {}}
        model['properties']['carrier_id'] = {
            'type': 'integer',
            'description': "carrier_id"
        }

        return model


class ResponseSchema(ResponseSchemaBase):

    def get_model_schema(self):
        model = {'type': 'object', 'properties': {}}
        model['properties']['version'] = {
            'type': 'string',
            'description': "version"
        }

        return model



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/1
import time
from flask import (
    g,
    request,
    current_app,
)
from yqn_project_cli.utils import error_mark
from yqn_project_cli.utils.restx import add_view_mth_resource
from yqn_project_cli.utils.core.exceptions import (
    APIException,
    PortalException,
    HTTPException
)
from yqn_project_cli.rpc import RPCConfigBase


def general_error_handler(e):
    if isinstance(e, (APIException, PortalException)) or current_app.config['DEBUG']:
        return e

    elif isinstance(e, HTTPException):
        # return APIException(errmsg=e.description, code=e.code)
        return APIException(msgDetail=e.description, code=e.code)

    else:
        error_mark(e)  # log it
        return PortalException()


# add resource type
def flask_restx_tail(plugin):
    import os
    import json
    import importlib
    from flask_restx import Namespace
    from yqn_project_cli.utils import load_module_from_pyfile
    from yqn_project_cli.utils.restx import DANamespace

    routes = json.load(
        open(os.path.join(plugin.app.project_config.project_base_dir,
                          'config/{}.json'.format(plugin.app.project_config.name)), 'r'))

    for route in routes['path_list']:
        # views = load_module_from_pyfile(os.path.join(api_path, module, 'views.py'))
        views = importlib.import_module('api.{}.views'.format(route['module']))
        # importlib.reload(views)

        for k, v in views.__dict__.items():
            if isinstance(v, DANamespace):

                add_view_mth_resource(v, getattr(getattr(views, route['view_cls']), route['view_mth']), route)

                plugin.add_namespace(v)

                break  # only allow one namespace

            elif isinstance(v, Namespace):
                raise ValueError('%s need inheriting from %s, rather than %s ' % (v, DANamespace, Namespace))


# decorator type
def flask_restx_tail_old(plugin):
    import os
    import importlib
    from flask_restx import Namespace
    from yqn_project_cli.utils import load_module_from_pyfile
    from yqn_project_cli.utils.restx import DANamespace

    api_path = os.path.join(plugin.app.project_config.project_base_dir, 'api')
    for module in os.listdir(api_path):
        if module.startswith(('_', '.')) \
                or not os.path.isdir(os.path.join(api_path, module)) \
                or not os.path.exists(os.path.join(api_path, module, 'views.py')):
            continue

        # views = load_module_from_pyfile(os.path.join(api_path, module, 'views.py'))
        views = importlib.import_module('api.{}.views'.format(module))
        # importlib.reload(views)

        for k, v in views.__dict__.items():
            if isinstance(v, DANamespace):
                plugin.add_namespace(v)

            elif isinstance(v, Namespace):
                raise ValueError('%s need inheriting from %s, rather than %s ' % (v, DANamespace, Namespace))


def close_rpc_services(e):
    for name, server in current_app.services.items():
        if name != (current_app.server_manager and current_app.server_manager[0]) and isinstance(server, RPCConfigBase):
            server.close()


def time_before_request():
    g.request_time = time.time()


def time_after_request(res):
    project_config = getattr(current_app, 'project_config', False)
    timer_logger_over_secs = getattr(project_config, 'timer_logger_over_secs', False)
    if project_config \
            and type(timer_logger_over_secs).__name__ in ('NoneType', 'int', 'float', 'bool')\
            and timer_logger_over_secs is not False \
            and not (isinstance(timer_logger_over_secs, (int, float) and timer_logger_over_secs < 0)) \
            and request.endpoint not in getattr(project_config, 'timer_logger_skip_endpoints', set()) \
            and isinstance(getattr(g, 'request_time', False), (int, float)):

        specs = str(request.full_path) + " with data: " + str(request.json or dict(request.data) or dict(request.args))

        response_time = time.time()
        wasted_time = response_time - g.request_time

        if timer_logger_over_secs is None \
                or isinstance(timer_logger_over_secs, (int, float)) and wasted_time >= timer_logger_over_secs:

            current_app.loggers(getattr(project_config, 'timer_logger_type', 'timer'))\
                .info("waste " + str(wasted_time) + " secs for " + specs)

    return res



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/24
import logging
from logging.handlers import TimedRotatingFileHandler
import os
from datetime import datetime
import json
from pathlib import Path


class LogFormatter(logging.Formatter):
    converter = datetime.fromtimestamp

    def __init__(self, app_name, app_id, log_type="Default"):
        self.app_name = app_name
        self.app_id = app_id
        self.log_type = log_type
        super().__init__()

    def format(self, record: logging.LogRecord):
        """
        {"log_timestamp": "2020/11/11 15:11:09",
         "trace_id": "",
         "remote_app_id": "unknownApp",
         "log_level": "info",
         "app_name": "appName",
         "thread_id": "13621",
         "log_type": "BIZ",
         "caller_id": "",
         "server_ip": "192.168.14.218",
         "client_ip": "0.0.0.0",
         "log_message": "TextPositionList END, UUID:92841272-82e3-4530-beef-bc822c4eb845 ,fileId:26472159",
         "exp": "",
         "app_id": "50010"}
        :param record:
        :return:
        """
        ct = self.converter(record.created)
        time_str = ct.strftime("%Y/%m/%d %H:%M:%S")
        log_dict = dict()
        log_dict['log_timestamp'] = time_str
        log_dict['trace_id'] = ''
        log_dict['remote_app_id'] = ''
        log_dict['log_level'] = record.levelname
        log_dict['app_name'] = self.app_name
        log_dict['thread_id'] = record.thread
        log_dict['log_type'] = self.log_type
        log_dict['caller_id'] = ''
        if 'SERVER_IP' in os.environ:
            log_dict['server_ip'] = os.environ['SERVER_IP']
        else:
            log_dict['server_ip'] = ''
        log_dict['client_ip'] = ''
        log_dict['log_message'] = record.msg
        log_dict['exp'] = ''
        log_dict['path_name'] = record.pathname
        log_dict['line_no'] = record.lineno
        log_dict['app_id'] = self.app_id
        return json.dumps(log_dict, ensure_ascii=False)


def get_logger(app_name, app_id, log_path, log_type='Default'):
    logger = logging.getLogger(app_name + "_" + log_type)
    logger.setLevel(logging.INFO)
    formatter = LogFormatter(app_name, app_id, log_type.upper())
    root_path = log_path
    Path(root_path).mkdir(parents=True, exist_ok=True)

    file_handler = logging.FileHandler(root_path + log_type.lower() + ".txt")
    # file_handler = TimedRotatingFileHandler(root_path + log_type.lower() + ".txt", when="D", backupCount=14)

    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.handlers = []
    logger.addHandler(file_handler)
    return logger


class Logger:
    def __init__(self, logger_config=None, log_type='Default'):
        assert isinstance(log_type, str), 'log type needs set to be str'

        self.logger_config = logger_config

        if self.logger_config is None:
            self.loggers = {}

        else:
            self.loggers = {log_type.lower(): get_logger(self.logger_config.app_name,
                                                         self.logger_config.app_id,
                                                         self.logger_config.log_path,
                                                         log_type)}

    def __call__(self, log_type: str):
        assert isinstance(log_type, str), 'log type needs set to be str'
        assert self.logger_config is not None, 'log must be init from config'

        if log_type.lower() not in self.loggers:
            self.loggers[log_type.lower()] = get_logger(
                self.logger_config.app_name,
                self.logger_config.app_id,
                self.logger_config.log_path,
                log_type)

        return self.loggers[log_type.lower()]



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/1
from odps import ODPS, options
from yqn_project_cli.rpc import RPCConfigBase
from yqn_project_cli.utils.decorators import time_it
from yqn_project_cli.utils import health_mark, warning_mark


class ODPSClient(RPCConfigBase):

    def __init__(self, **kwargs):
        super(ODPSClient, self).__init__(**kwargs)
        self.odps_access_id = None
        self.odps_secret_access_key = None
        self.odps_project = None
        self.odps_endpoint = None

    def connect(self, **kwargs):
        self.odps_access_id = self.server_manager.get_value('odps_access_id') if self.use_conf_management \
            else self.client_config.odps_access_id

        self.odps_secret_access_key = \
            self.server_manager.get_value('odps_secret_access_key') if self.use_conf_management \
                else self.client_config.odps_secret_access_key

        self.odps_project = self.server_manager.get_value('odps_project') if self.use_conf_management \
            else self.client_config.odps_project

        self.odps_endpoint = self.server_manager.get_value('odps_endpoint') if self.use_conf_management \
            else self.client_config.odps_endpoint

        self.connector = ODPS(self.odps_access_id, self.odps_secret_access_key, self.odps_project, self.odps_endpoint)

        options.tunnel.use_instance_tunnel = True
        options.tunnel.limit_instance_tunnel = False
        options.allow_antique_date = True

    def close(self, **kwargs):
        pass

    def reconnect(self, **kwargs):
        pass

    def __str__(self):
        return self.odps_project

    def check_table_info(self, tb_name):
        tb = self.connector.get_table(tb_name)
        print(tb.schema)

    @time_it
    def execute_sql(self, sql, print_info=False):
        with self.connector.execute_sql(sql).open_reader() as r:

            if print_info:
                print('sql:', sql)

                if r.count:
                    health_mark('count: %s \n %s' % (r.count, r[0]))

                else:
                    warning_mark('None of Match')

            return r
